import os.path as osp
import sys
import tkinter as tk
from tkinter import filedialog as fd

import numpy as np
import cv2
from PIL import Image, ImageTk

from .util import load_nii, wwwc
from .predict import predict


class MenuBar(tk.Menu):
    def __init__(self, parent, load_scan, load_label, run_predict, choose_model_path):
        tk.Menu.__init__(self, parent)

        fileMenu = tk.Menu(self, tearoff=False)
        self.add_cascade(label="File", underline=0, menu=fileMenu)
        fileMenu.add_command(label="Load Scan", underline=1, command=load_scan)
        fileMenu.add_command(label="Load Label", underline=1, command=load_label)
        fileMenu.add_command(label="Model Path", underline=1, command=choose_model_path)
        fileMenu.add_command(label="Predict", underline=1, command=run_predict)
        fileMenu.add_command(label="Exit", underline=1, command=self.quit)

    def quit(self):
        sys.exit(0)


class App(tk.Tk):
    def __init__(self):
        tk.Tk.__init__(self)
        menubar = MenuBar(
            self,
            load_scan=self.load_scan,
            load_label=self.load_label,
            run_predict=self.run_predict,
            choose_model_path=self.choose_model_path,
        )
        self.config(menu=menubar)
        self.scan = None  # 扫描
        self.label = None  # 标签
        self.curr_idx = None  # 当前片层id
        self.model_path = None  # 模型best_model路径，默认值是 ./output/bisenet-4epoch/best_model
        self.transparency = 0.5  # 叠加时标签的透明度

        self.bind("<Control-q>", self.quit)
        self.bind("<w>", lambda ev: self.turn(1))
        self.bind("<s>", lambda ev: self.turn(-1))
        # self.bind("<MouseWheel>", self.scroll)

        self.bind("<MouseWheel>", self.scroll)
        self.bind("<Button-4>", lambda ev: self.turn(1))
        self.bind("<Button-5>", lambda ev: self.turn(-1))
        self.bind("<Button-1>", lambda ev: self.master)

    def run_predict(self):
        """跑推理，设置label"""
        if self.model_path is None:
            self.choose_model_path()
        if self.scan is None:
            self.load_scan()
        
        if self.model_path is None:
            print("Model path is not set , cannot run predict")
        if self.scan is None:
            print("Scan is not loaded, cannot run predict")
        
        self.label = predict(
            self.scan,
            self.model_path,
        )
        self.turn(0)

    def scroll(self, ev):
        print(ev, ev.delta)
        self.turn(ev.delta // 120)

    def choose_model_path(self):
        model_path = fd.askdirectory(
            title="Choose model path",
            initialdir=".",
        )
        if model_path is None or len(model_path) == 0 or not osp.exists(model_path):
            print("Scan path don't exist")
            self.model_path = None
            return
        self.model_path = model_path

    def load_scan(self):
        filetypes = (("CT Scan", "*.nii, *.nii.gz"),)

        scan_path = fd.askopenfilename(
            title="Open a abdominal CT scan",
            initialdir="/data/liver/Task03_Liver/imagesTr",
            filetypes=filetypes,
        )
        if not osp.exists(scan_path):
            print("Scan path don't exist")
            return

        scan = load_nii(scan_path)
        scan = wwwc(scan)
        self.scan = scan.astype("uint8")

        if self.curr_idx is None:
            self.turn_to(self.scan.shape[0] // 2)

    def load_label(self):
        filetypes = (("Segmentation Mask", "*.nii, *.nii.gz"),)

        label_path = fd.askopenfilename(
            title="Open a segmentation mask",
            initialdir="/data/liver/Task03_Liver/labelsTr/",
            filetypes=filetypes,
        )
        if not osp.exists(label_path):
            print("Label path don't exsist")
            return

        label = load_nii(label_path)
        self.label = label.astype("uint8")

        if self.curr_idx is None:
            self.turn_to(self.label.shape[0] // 2)
        else:
            self.turn_to(self.curr_idx)

    def turn(self, delta):
        if self.curr_idx is None:
            return
        self.turn_to(self.curr_idx + delta)

    def turn_to(self, turn_to_idx):

        if self.scan is None and self.label is None:
            print("Both None!!")
            return
        max_idx = self.scan.shape[0] if self.scan is not None else self.label.shape[0]
        if turn_to_idx < 0 or turn_to_idx >= max_idx:
            print(turn_to_idx, " out of range")
            return

        if self.scan is None:
            scan_color = np.zeros((self.label.shape[1], self.label.shape[2], 3), dtype="uint8")
        else:
            scan_color = cv2.cvtColor(self.scan[turn_to_idx], cv2.COLOR_GRAY2RGB)

        if self.label is None:
            label_color = np.zeros_like(scan_color, dtype="uint8")
        else:
            label_slice = self.label[turn_to_idx, :, :]
            label_color = np.zeros((self.label.shape[1], self.label.shape[2], 3), dtype="uint8")
            label_color[label_slice == 1, :] = [255, 0, 0]
            # label_color[label_slice == 2, :] = [0, 255, 0]
        # print(
        #     "blending",
        #     scan_color.shape,
        #     scan_color.dtype,
        #     label_color.shape,
        #     label_color.dtype,
        #     self.transparency,
        # )

        # image = qimage2ndarray.array2qimage(image)
        if self.label is not None:
            image = cv2.addWeighted(scan_color, 1, label_color, 1 - self.transparency, 0)
        else:
            image = scan_color
        # print("image.shape", image.shape)

        img = Image.fromarray(image)
        render = ImageTk.PhotoImage(img)
        img = tk.Label(self, image=render)
        img.image = render
        img.place(x=0, y=0)

        self.curr_idx = turn_to_idx

    def quit(self, ev):
        sys.exit(0)


def main():
    app = App()
    app.title("Liver Segmentation")
    app.geometry("512x512")
    app.mainloop()


if __name__ == "__main__":
    main()
